"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseSimpleIdentifierList = parseSimpleIdentifierList;
exports.buildVariableContext = buildVariableContext;
exports.getArrayDimensions = getArrayDimensions;
exports.bindVariableContext = bindVariableContext;
exports.resolveDimensions = resolveDimensions;
function parseSimpleIdentifierList(ctx) {
    const result = [];
    if (!ctx) {
        return result;
    }
    ctx.ID_list().forEach((node) => {
        result.push(node.getText());
    });
    return result;
}
function buildVariableContext(names, values) {
    if (names.length !== values.length) {
        throw new Error("Names and values must have the same length");
    }
    const context = {};
    for (let i = 0; i < names.length; i++) {
        const bindContext = bindVariableContext(names[i], getArrayDimensions(values[i]), values[i]);
        for (const key in bindContext) {
            if (bindContext[key] !== null) {
                context[key] = bindContext[key];
            }
        }
    }
    return context;
}
function getArrayDimensions(value) {
    if (Array.isArray(value)) {
        return [value.length, ...getArrayDimensions(value[0])];
    }
    return [];
}
function bindVariableContext(variableName, dimensions, values) {
    const context = {};
    const resolved = resolveDimensions(variableName, dimensions);
    for (const variable of resolved) {
        try {
            context[variable] = parseVariable(values, variable.replace(variableName, ""));
        }
        catch {
            context[variable] = null;
        }
    }
    return context;
}
function resolveDimensions(variableName, dimensions) {
    return internalResolveDimensions([variableName], dimensions, 0);
}
// reference MUST be similar to [0][1]
function parseVariable(value, reference) {
    const parts = reference
        .split("[")
        .map((part) => part.replace("]", ""))
        .filter((part) => part !== "")
        .map((part) => parseInt(part));
    return getReferenceValueInternal(value, parts);
}
function getReferenceValueInternal(value, reference) {
    if (reference.length === 0) {
        return value;
    }
    if (!Array.isArray(value)) {
        throw new Error("INTERNAL ERROR! Reference is invalid");
    }
    return getReferenceValueInternal(value[reference[0]], reference.slice(1));
}
function internalResolveDimensions(variables, dimensions, layer) {
    if (layer >= dimensions.length) {
        return variables;
    }
    const result = [];
    for (let i = 0; i < dimensions[layer]; i++) {
        for (const variable of variables) {
            result.push(`${variable}[${i}]`);
        }
    }
    return internalResolveDimensions(result, dimensions, layer + 1);
}
